/*

Remote Controlled Fan Regulator.
********************************

Compiler: avr-gcc (WinAVR-20090313)
Project Manager/IDE: AVR Studio 4.17 Build 666
Other Lib: eXtreme NEC Decoder.

Hardware:
	INT0 - IR Receiver
	INT1 - Zero Crossing Detector.
	PD5 - Triac Control.
	PB2 - Manual Switch

For
	ATmega8 @ 16MHz

Fuse: 
	HIGH=0xC9 LOW=0xFF

	Copyright (c) 2008-2010
	eXtreme Electronics, India

                                     NOTICE
									--------
NO PART OF THIS WORK CAN BE COPIED, DISTRIBUTED OR PUBLISHED WITHOUT A
WRITTEN PERMISSION FROM EXTREME ELECTRONICS INDIA. THE LIBRARY, NOR ANY PART
OF IT CAN BE USED IN COMMERCIAL APPLICATIONS. IT IS INTENDED TO BE USED FOR
HOBBY, LEARNING AND EDUCATIONAL PURPOSE ONLY. IF YOU WANT TO USE THEM IN 
COMMERCIAL APPLICATION PLEASE WRITE TO THE AUTHOR.

*/


#include <avr/io.h>
#include <util/delay_basic.h>
#include <avr/interrupt.h>

#include "remote.h"
#include "rckeys.h"

#define FAN_POWER_LED_PORT PORTD
#define FAN_POWER_LED_DDR DDRD
#define FAN_POWER_LED_BIT 7

#define POWER_LED_ON() 	FAN_POWER_LED_PORT&=(~(1<<FAN_POWER_LED_BIT))
#define POWER_LED_OFF()	FAN_POWER_LED_PORT|=(1<<FAN_POWER_LED_BIT)

uint8_t table[10]={141,125,109,94,78,62,47,31,16,1};

uint8_t speed=0;
uint8_t fan_on=0;

void Initialize()
{
	FAN_POWER_LED_DDR|=0B10000000;

	POWER_LED_OFF();

	DDRC|=0b00111111;	//Seven segment
	DDRB|=0b00000010;	//Middle segment G

	Display(0);

	RemoteInit();

	//Initialize the zero crossing detector INT(1)
	
	MCUCR|=((1<<ISC11)|(1<<ISC10));		//INT in Rising edge
	GICR|=(1<<INT1);					//Enable INT1

	//Output
	DDRD|=(1<<PD5);
	PORTD|=(1<<PD5);	//High = TRIAC Off


	//Set Timer 2
	TCCR2|=(1<<WGM21);	//CTC
	TIMSK|=(1<<OCIE2);	//Enable OCI2

	sei();

}
/*

Zero Crossing Detect.

*/
ISR(INT1_vect)
{
	if(!fan_on) 
	{
		PORTD|=(1<<PD5);	//High = TRIAC Off
		return;
	}
		
	if(speed==9)
	{
		PORTD&=(~(1<<PD5)); //low = TRIAC ON
		return;
	}
		
	PORTD|=(1<<PD5);	//High = TRIAC Off

	OCR2=table[speed];

	TCNT2=0x00;

	TCCR2|=((1<<CS22)|(1<<CS21)|(1<<CS20));	//Start Timer prescaler =1024

}

/*

Timer2 Compare ISR

*/
ISR(TIMER2_COMP_vect)
{
	PORTD&=(~(1<<PD5)); //low = TRIAC ON

	TCCR2&=(~((1<<CS22)|(1<<CS21)|(1<<CS20)));	//Stop Timer
}

/*

Simple Wait Function

*/
void Wait()
{
	char i;

	for(i=0;i<100;i++)
		_delay_loop_2(0);
}

/*

Displays a number in Seven Seg Display

*/
void Display(uint8_t num)
{
	if(num>9)
		return;
	
	switch (num)
	{
		case 0:
			PORTC=0B00000000;
			PORTB|=0B00000010;
			break;
		case 1:
		    //      xxfedcba
			PORTC=0B00111001;
			PORTB|=0B00000010;
			break;
		case 2:
			//      xxfedcba
			PORTC=0B00100100;
			PORTB&=(~(0B00000010));
			break;
		case 3:
			//      xxfedcba
			PORTC=0B00110000;
			PORTB&=(~(0B00000010));
			break;
			break;
		case 4:
			//      xxfedcba
			PORTC=0B00011001;
			PORTB&=(~(0B00000010));
			break;
		case 5:
			//      xxfedcba
			PORTC=0B00010010;
			PORTB&=(~(0B00000010));
			break;
		case 6:
			//      xxfedcba
			PORTC=0B00000010;
			PORTB&=(~(0B00000010));
			break;
		case 7:
			//      xxfedcba
			PORTC=0B00111000;
			PORTB|=0B00000010;
			break;
		case 8:
			//      xxfedcba
			PORTC=0B00000000;
			PORTB&=(~(0B00000010));
			break;
		case 9:
			//      xxfedcba
			PORTC=0B00010000;
			PORTB&=(~(0B00000010));
			break;
	}
}
			
void main()
{

	uint8_t cmd; //Command received from remote

	Initialize();

	while(1)
	{
		//Get Command For the Remote Control
		cmd=GetRemoteCmd(1);

		//Now process the command

		//UP Key
		if(cmd==RC_UP)
		{
			if(speed<9)
				speed++;

		}

		//DOWN Key
		if(cmd==RC_DOWN)
		{
			if(speed>0)
				speed--;
		}

		//Enter Key
		if(cmd==RC_ENTER)
		{
			if(fan_on)
			{
				POWER_LED_OFF();
				fan_on=0;	//Turn Off
			}
			else
			{
				POWER_LED_ON();
				fan_on=1;	//Turn On
			}
		}

		Display(speed);

	}
}
